#ifdef __QNX__

#include <tv.h>

#include <string.h>
#include <signal.h>
#include <sys/dev.h>
#include <qnxvideo.h>

char ConsoleMode=True;

char DefaultRadioButton=0x07;

char TerminalStatus=False;
ushort QNXCursorState=0x0607;
ushort QNXCursorLastX=0;
ushort QNXCursorLastY=0;
ushort QNXScreenSizeX=80;
ushort QNXScreenSizeY=25;
char   QNXResizeStatus=0;

char QNXMouseStatus;
char QNXButtonCount;

void SignalStubber(int)
{
   // just do nothing ... ;)
}

void QNXInitTerminal()
{
   _dev_info_entry DeviceInfo;

   if (TerminalStatus==True)
      return;
   if (term_load()<0)
   {
      printf("\nCouldn't get terminal definitions !\n");
      _exit(1);
   }
   term_video_on();
   term_resize_on();
   term_box_off();
   term_flush();
   
   dev_info(fileno(stdin), &DeviceInfo);
   if (strcmp(DeviceInfo.driver_type, "console")==0)
   {
      ConsoleMode=True;
      DefaultRadioButton=0x07;
   }
   else
   {
      ConsoleMode=False;
      DefaultRadioButton=0xFE;
   }

   dev_mode(fileno(stdin), _DEV_OPOST, _DEV_OPOST);

   signal(SIGINT,  SignalStubber);
   
   TerminalStatus=True;
}

void QNXTerminalSuspend()
{
}

void QNXTerminalResume()
{
}

void QNXSetScreenMode(ushort mode)
{
   struct winsize win;

   win.ws_col=char(mode>>8);
   win.ws_row=char(mode);
   win.ws_xpixel=0;
   win.ws_ypixel=0;
   
   ioctl(fileno(stdin), TIOCSWINSZ, &win);

   QNXScreenSizeX=QNXGetScreenCols();
   QNXScreenSizeY=QNXGetScreenRows();
   
   term_relearn_size();
   
   QNXClearScreen(0, 0);
}

ushort QNXGetScreenRows()
{
   struct winsize win;

   ioctl(fileno(stdin), TIOCGWINSZ, &win);
   QNXScreenSizeY=win.ws_row;

   return win.ws_row;
}

ushort QNXGetScreenCols()
{
   struct winsize win;

   ioctl(fileno(stdin), TIOCGWINSZ, &win);
   QNXScreenSizeX=win.ws_col;

   return win.ws_col;
}

void QNXClearScreen(int, int)
{
   if (TerminalStatus==False)
      QNXInitTerminal();
   term_color(TERM_WHITE | TERM_BLACK_BG);
   term_clear(TERM_CLS_SCR);
   term_flush();
}

ushort QNXGetCursorState()
{
   return QNXCursorState;
}

void QNXSetCursorState(ushort State)
{
   char* CursorString;

   QNXCursorState=State;

   CursorString=cursor_normal;
   if (char(State>>8)==0x20)
   {
      CursorString=cursor_invisible;
   }
   if (char(State>>8)==0x00)
   {
      CursorString=cursor_visible;
   }
   __putp(CursorString);
   term_flush();
}

void QNXSetCursorPos(int row, int col)
{
   if (TerminalStatus==False)
      QNXInitTerminal();

      QNXCursorLastX=col;
      QNXCursorLastY=row;
      term_cur(row, col);
      term_flush();
}

void QNXMouseStart()
{
   if (TerminalStatus==False)
      QNXInitTerminal();
   if (term_mouse_on()==0)
   {
      QNXButtonCount=2;
   }
   else
   {
      QNXButtonCount=0;
   }
}

void QNXMouseStop()
{
   if (TerminalStatus==False)
      QNXInitTerminal();
   term_mouse_off();
}

void QNXMouseShow()
{
   if (TerminalStatus==False)
      QNXInitTerminal();
   QNXMouseStatus=1;
   term_mouse_move(-1, -1);
   term_flush();
}

void QNXMouseHide()
{
   if (TerminalStatus==False)
      QNXInitTerminal();
   QNXMouseStatus=0;
   term_mouse_hide();
   term_flush();
}

void QNXCopyLineToScreen(int y, int x, int len, uchar* Buffer)
{
   if (ConsoleMode==False)
   {
      for (int EachSymbol=0; EachSymbol<len; EachSymbol++)
      {
         switch (*(Buffer+EachSymbol*2))
	 {
	    case 0x00:
	    case 0x01:
	    case 0x02:
	    case 0x03:
	    case 0x04:
	    case 0x05:
	    case 0x06:
	    case 0x07:
	    case 0x08:
	    case 0x09:
	    case 0x0A:
	    case 0x0B:
	    case 0x0C:
	    case 0x0D:
	    case 0x1B:
	               *(Buffer+EachSymbol*2)=0x20;
	               break;
	 }
      }
   }

   if (ConsoleMode==False)				// Photon bug elimination ;)
   {
      if (y>QNXScreenSizeY/2)
	  {
	     term_cur(0, 0);
	  }
	  else
	  {
	     term_cur(QNXScreenSizeY-1, 0);
	  }
	  term_flush();
	  
      term_cur(y, x);
      term_flush();
   }
	  
   term_restore_image(y, x, (char*)Buffer, len);
   term_flush();

   if (ConsoleMode==False)
   {
      QNXSetCursorPos(QNXCursorLastY, QNXCursorLastX);
   }
}

int QNXKeyHit()
{
   int TempKeyHit;
   int TempKeyRead;
   
   TempKeyHit=kbhit();
   
   if (!TempKeyHit)
   {
      __nodelay(0, 1);
      TempKeyRead=term_key();
      if (TempKeyRead==0x0300)
      {
	 if (QNXResizeStatus==0)
	 {
            __nodelay(0, 0);
	    __ungetch(0x0300);
	    QNXResizeStatus=1;
	    return 1;
	 }
      }
      __nodelay(0, 0);
   }
   
   return TempKeyHit;
}

ushort QNXGetKey()
{
   return term_key();
}

#endif
